<?php

/******************************************************************************/
// sitePath
/******************************************************************************/
function sitePath()
{
	return substr(substr(__FILE__, 0, strrpos(__FILE__,'wp-content')), 0, -1);
	//		$uploadsPath = 	substr($uploadsPath, 0, -1);
}
/******************************************************************************/
// downloadRemoteFile
/******************************************************************************/
function downloadRemoteFile($remoteFile)
{
	try
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $remoteFile); 
		curl_setopt($ch,CURLOPT_RETURNTRANSFER,1);
		curl_setopt($ch,CURLOPT_CONNECTTIMEOUT,5);
		curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
		$resp = curl_exec($ch); 
		curl_close($ch);
		$tmpfname = tempnam ( '/tmp', 'gpx' );
		
		$fp = fopen($tmpfname, "w");
		fwrite($fp, $resp);
		fclose($fp);
		
		return $tmpfname;
	} catch (Exception $e) {
		return '';
	}
}
/******************************************************************************/
// toRadians
/******************************************************************************/
function toRadians($degrees)
{
	return (float)($degrees * 3.1415926535897932385 / 180);
}
/******************************************************************************/
// date_getDecimals
/******************************************************************************/
function date_getDecimals($date)
{
	if (preg_match('(\.([0-9]{2})Z?)', $date, $matches))
	{
		return (float)((float)$matches[1] / 100);
	}
	else
	{
		return 0;
	}
}
/******************************************************************************/
// my_date_diff
/******************************************************************************/
function my_date_diff($old_date, $new_date) {

	$t1 = strtotime($new_date);
	$t2 = strtotime($old_date);
	
	// milliceconds fix
	$t1 += date_getDecimals($new_date);
	$t2 += date_getDecimals($old_date);

	$offset = (float)($t1 - $t2);
  
	//echo "$offset = $new_date - $old_date; ".strtotime($new_date)." ".strtotime($old_date)." <br />";
  
  return $offset;
}
/******************************************************************************/
// calculateDistance
/******************************************************************************/
function calculateDistance($lat1,$lon1,$ele1,$lat2,$lon2,$ele2)
{
	$alpha = (float)sin((float)toRadians((float) $lat2 - (float) $lat1) / 2);
	$beta = (float)sin((float)toRadians((float) $lon2 - (float) $lon1) / 2);
	//Distance in meters
	$a = (float) ( (float)$alpha * (float)$alpha) +  (float) ( (float)cos( (float)toRadians($lat1)) * (float)cos( (float)toRadians($lat2)) * (float)$beta * (float)$beta );
	$dist = 2 * 6369628.75 * (float)atan2((float)sqrt((float)$a), (float)sqrt(1 - (float) $a));
	$d = (float)sqrt((float)pow((float)$dist, 2) + pow((float) $lat1 - (float)$lat2, 2));	
	return sqrt((float)pow((float)$ele1-(float)$ele2,2)+(float)pow((float)$d,2));
}
/******************************************************************************/
// parseXml
/******************************************************************************/
function parseXml($filePath, $gpxOffset)
{

	$points = array();
	$gpx = simplexml_load_file($filePath);	
	
	if($gpx === FALSE) 
		return;
	
	$gpx->registerXPathNamespace('10', 'http://www.topografix.com/GPX/1/0'); 
	$gpx->registerXPathNamespace('11', 'http://www.topografix.com/GPX/1/1'); 
	$gpx->registerXPathNamespace('gpxx', 'http://www.garmin.com/xmlschemas/GpxExtensions/v3'); 		
	
	$nodes = $gpx->xpath('//trkpt | //10:trkpt | //11:trkpt');
	
	if ( count($nodes) > 0 )	
	{
	
		$lastLat = 0;
		$lastLon = 0;
		$lastEle = 0;
		$lastTime = 0;
		$dist = 0;
		$lastOffset = 0;
		$speedBuffer = array();
	
		// normal case
		foreach($nodes as $trkpt)
		{ 
			$lat = $trkpt['lat'];
			$lon = $trkpt['lon'];
			$ele = $trkpt->ele;
			$time = $trkpt->time;
			$speed = (float)$trkpt->speed;

			if ($lastLat == 0 && $lastLon == 0)
			{
				//Base Case
				array_push($points, array((float)$lat,(float)$lon,(float)round($ele,2),(float)round($dist,2), 0 ));
				$lastLat=$lat;
				$lastLon=$lon;
				$lastEle=$ele;				
				$lastTime=$time;	
			}
			else
			{
				//Normal Case
				$offset = calculateDistance((float)$lat, (float)$lon, (float)$ele, (float)$lastLat, (float)$lastLon, (float)$lastEle);
				$dist = $dist + $offset;
				
				if ($speed == 0)
				{
					$datediff = (float)my_date_diff($lastTime,$time);
					//echo "------------$time-------$lastTime-----";
					//echo "------------$datediff------------";
					if ($datediff>0)
					{
						$speed = $offset / $datediff;
					}
				}
				
				array_push($speedBuffer, $speed);
				
				if (((float) $offset + (float) $lastOffset) > $gpxOffset)
				{
					//Bigger Offset -> write coordinate
					
					$avgSpeed = 0;
					
					foreach($speedBuffer as $s)
					{ 
						$avgSpeed += $s;
					}
					
					$avgSpeed = $avgSpeed / count($speedBuffer);
					$speedBuffer = array();
					
					$lastOffset=0;
					array_push($points, array(
												(float)$lat,
												(float)$lon,
												(float)round($ele,1),
												(float)round($dist,1), 
												(float)round($avgSpeed,1) 
											)
								);
				}
				else
				{
					//Smoller Offset -> continue..
					$lastOffset = (float) $lastOffset + (float) $offset ;
				}
			}
			$lastLat=$lat;
			$lastLon=$lon;
			$lastEle=$ele;
			$lastTime=$time;	
		}
		unset($nodes);
	
	}
	else
	{
	
		$nodes = $gpx->xpath('//gpxx:rpt');
	
		if ( count($nodes) > 0 )	
		{
		
			$lastLat = 0;
			$lastLon = 0;
			$lastEle = 0;
			$dist = 0;
			$lastOffset = 0;
		
			// Garmin case
			foreach($nodes as $rpt)
			{ 
			
				$lat = $rpt['lat'];
				$lon = $rpt['lon'];
				if ($lastLat == 0 && $lastLon == 0)
				{
					//Base Case
					array_push($points, array((float)$lat,(float)$lon,null,null));
					$lastLat=$lat;
					$lastLon=$lon;
				}
				else
				{
					//Normal Case
					$offset = calculateDistance($lat, $lon, 0,$lastLat, $lastLon, 0);
					$dist = $dist + $offset;
					if (((float) $offset + (float) $lastOffset) > $gpxOffset)
					{
						//Bigger Offset -> write coordinate
						$lastOffset=0;
						array_push($points, array((float)$lat,(float)$lon,null,null));
					}
					else
					{
						//Smoller Offset -> continue..
						$lastOffset= (float) $lastOffset + (float) $offset;
					}
				}
				$lastLat=$lat;
				$lastLon=$lon;
			}
			unset($nodes);
		}
		else
		{	
			echo "Empty Gpx or not supported File!";
		}
	}
	unset($gpx);
	return $points;
}
/******************************************************************************/
// getPoints
/******************************************************************************/
function getPoints($gpxPath,$gpxOffset = 10, $donotreducegpx)
{

	$points = array();
	$dist=0;
	
	$lastLat=0;
	$lastLon=0;
	$lastEle=0;
	$lastOffset=0;
			
	if (file_exists($gpxPath))
	{
		$points = parseXml($gpxPath, $gpxOffset);			
	}
	else
	{
		array_push($points, array((float)0,(float)0,(float)0,(float)0));
		echo "File $gpxPath not found!";
	}
	
	// reduce the points to around 200 to speedup
	if ( $donotreducegpx != true)
	{
		$count=sizeof($points);
		if ($count>200)
		{
			$f = round($count/200);
			if ($f>1)
				for($i=$count;$i>0;$i--)
					if ($i % $f != 0)
						unset($points[$i]);
		}		
	}
	return $points;
}

/******************************************************************************\
* parseGPX
In :
	$gpx : url du fichier gpx
	$uom : unité : miles / meter
	$showSpeed : récupère ou non la vitesse 
	$uomspeed : unité de la vitesse
	$showW : récupère ou non les Waypoints 
Out :
	$points_maps : tableau des pts [lat,lon],...
	$points_graph : tableau : [dist,elev,speed],...
	$waypoints : tableau [lat,lon,...],...
/******************************************************************************/
function parseGPX($gpx) {
	$sitePath = sitePath();
	$gpx = trim($gpx);

	if (strpos($gpx, "http://") !== 0)
	{
		$gpx = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $gpx);
		$gpx = $sitePath . $gpx;
	}
	else
	{
		$gpx = downloadRemoteFile($gpx);
	}

	if ($gpx == '')
	{
		return "No gpx found";
	}

	$points = getPoints( $gpx, $pointsoffset, $donotreducegpx);
	$points_maps = '';
	$points_graph = '';
	$waypoints = '';

	foreach ($points as $p) {
		$points_maps .= '['.(float)$p[0].','.(float)$p[1].'],';

		$_dist = $p[3];
		$_ele = $p[2];
	
		if ($uom == '1')
		{
			// Miles and feet			
			$_dist *= 0.000621371192;
			$_ele *= 3.2808399;
		} else if ($uom == '2')
		{
			// meters / kilometers
			$_dist = $_dist/1000;
		}
	
		if ($showSpeed == true)
		{
	
			$_speed = $p[4]; // default m/s
		
			if ($uomspeed == '2') // miles/h
			{
				$_speed *= 2.2369362920544025;
			} 
			else if ($uomspeed == '1') // km/h
			{
				$_speed *= 3.6;
			} 				
	
			$points_graph .= '['.$_dist.','.$_ele.','.$_speed.'],';	
		}
		else
		{
			$points_graph .= '['.$_dist.','.$_ele.'],';	
		}
	}
	
	if ($showW == true)
	{
		$wpoints = getWayPoints($gpx);
		foreach ($wpoints as $p) {
			$waypoints .= '['.(float)$p[0].','.(float)$p[1].',\''.unescape($p[4]).'\',\''.unescape($p[5]).'\',\''.unescape($p[7]).'\'],';
		}
	}

	$p="/,$/";
	$points_maps = preg_replace($p, "", $points_maps);
	$points_graph = preg_replace($p, "", $points_graph);			
	$waypoints = preg_replace($p, "", $waypoints);

	if (preg_match("/^(\[0,0\],?)+$/", $points_graph)) 
	{		
		$points_graph = "";	
	}
	
	return $points_graph;

	
}
?>
